@@ -18,11 +18,13 @@ using namespace alglib;
 const char *fmt_str = "%-29s %s\n";
 
 //
-// Functions and flag variable for test "issue_505"
+// Flag variables
 //
 bool issue505_passed = true;
 bool issue478_passed = true;
 bool issue528_passed = true;
+bool issue591_passed = true;
+bool issue594_passed = true;
 
 //
 // Service datatypes
@@ -44,17 +46,15 @@ typedef struct
     alglib_impl::ae_shared_pool pool;
 } seedrec;
 
-ae_bool _innerrec_init(void* _p, alglib_impl::ae_state *_state, ae_bool make_automatic)
+void _innerrec_init(void* _p, alglib_impl::ae_state *_state)
 {
     innerrec *p = (innerrec*)_p;
     alglib_impl::ae_touch_ptr((void*)p);
-    if( !alglib_impl::ae_vector_init(&p->i1val, 0, alglib_impl::DT_INT, _state, make_automatic) )
-        return ae_false;
-    return ae_true;
+    alglib_impl::ae_vector_init(&p->i1val, 0, alglib_impl::DT_INT, _state);
 }
 
 
-ae_bool _innerrec_init_copy(void* _dst, void* _src, alglib_impl::ae_state *_state, ae_bool make_automatic)
+void _innerrec_init_copy(void* _dst, void* _src, alglib_impl::ae_state *_state)
 {
     innerrec *dst = (innerrec*)_dst;
     innerrec *src = (innerrec*)_src;
@@ -62,9 +62,7 @@ ae_bool _innerrec_init_copy(void* _dst, void* _src, alglib_impl::ae_state *_stat
     dst->rval = src->rval;
     dst->ival = src->ival;
     dst->bval = src->bval;
-    if( !alglib_impl::ae_vector_init_copy(&dst->i1val, &src->i1val, _state, make_automatic) )
-        return ae_false;
-    return ae_true;
+    alglib_impl::ae_vector_init_copy(&dst->i1val, &src->i1val, _state);
 }
 
 
@@ -84,28 +82,22 @@ void _innerrec_destroy(void* _p)
 }
 
 
-ae_bool _seedrec_init(void* _p, alglib_impl::ae_state *_state, ae_bool make_automatic)
+void _seedrec_init(void* _p, alglib_impl::ae_state *_state)
 {
     seedrec *p = (seedrec*)_p;
     alglib_impl::ae_touch_ptr((void*)p);
-    if( !_innerrec_init(&p->recval, _state, make_automatic) )
-        return ae_false;
-    if( !alglib_impl::ae_shared_pool_init(&p->pool, _state, make_automatic) )
-        return ae_false;
-    return ae_true;
+    _innerrec_init(&p->recval, _state);
+    alglib_impl::ae_shared_pool_init(&p->pool, _state);
 }
 
 
-ae_bool _seedrec_init_copy(void* _dst, void* _src, alglib_impl::ae_state *_state, ae_bool make_automatic)
+void _seedrec_init_copy(void* _dst, void* _src, alglib_impl::ae_state *_state)
 {
     seedrec *dst = (seedrec*)_dst;
     seedrec *src = (seedrec*)_src;
     dst->bval = src->bval;
-    if( !_innerrec_init_copy(&dst->recval, &src->recval, _state, make_automatic) )
-        return ae_false;
-    if( !alglib_impl::ae_shared_pool_init_copy(&dst->pool, &src->pool, _state, make_automatic) )
-        return ae_false;
-    return ae_true;
+    _innerrec_init_copy(&dst->recval, &src->recval, _state);
+    alglib_impl::ae_shared_pool_init_copy(&dst->pool, &src->pool, _state);
 }
 
 
@@ -1208,7 +1200,7 @@ int main()
             printf(fmt_str, "* issue 478", "OK (skipped in 32-bit mode)");
             fflush(stdout);
         }
-            
+        
         //
         // Testing issue #528 (http://bugs.alglib.net/view.php?id=528)
         // in shared pool and smart pointer which leak memory.
@@ -1234,8 +1226,8 @@ int main()
             alloc_cnt = alglib_impl::_alloc_counter;
             alglib_impl::ae_state_init(&_alglib_env_state);
             alglib_impl::ae_frame_make(&_alglib_env_state, &_frame_block);
-            alglib_impl::ae_shared_pool_init(&pool, &_alglib_env_state, ae_true);
-            _seedrec_init(&seed, &_alglib_env_state, ae_true);
+            alglib_impl::ae_shared_pool_init(&pool, &_alglib_env_state);
+            _seedrec_init(&seed, &_alglib_env_state);
             alglib_impl::ae_shared_pool_set_seed(&pool, &seed, sizeof(seed), _seedrec_init, _seedrec_init_copy, _seedrec_destroy, &_alglib_env_state);
             alglib_impl::ae_state_clear(&_alglib_env_state);
             issue528_passed = issue528_passed && (alloc_cnt==alglib_impl::_alloc_counter);
@@ -1244,9 +1236,9 @@ int main()
             alloc_cnt = alglib_impl::_alloc_counter;
             alglib_impl::ae_state_init(&_alglib_env_state);
             alglib_impl::ae_frame_make(&_alglib_env_state, &_frame_block);
-            alglib_impl::ae_smart_ptr_init(&ptr0, (void**)&p0, &_alglib_env_state, ae_true);
-            alglib_impl::ae_shared_pool_init(&pool, &_alglib_env_state, ae_true);
-            _seedrec_init(&seed, &_alglib_env_state, ae_true);
+            alglib_impl::ae_smart_ptr_init(&ptr0, (void**)&p0, &_alglib_env_state);
+            alglib_impl::ae_shared_pool_init(&pool, &_alglib_env_state);
+            _seedrec_init(&seed, &_alglib_env_state);
             alglib_impl::ae_shared_pool_set_seed(&pool, &seed, sizeof(seed), _seedrec_init, _seedrec_init_copy, _seedrec_destroy, &_alglib_env_state);
             alglib_impl::ae_shared_pool_retrieve(&pool, &ptr0, &_alglib_env_state);
             alglib_impl::ae_state_clear(&_alglib_env_state);
@@ -1256,10 +1248,10 @@ int main()
             alloc_cnt = alglib_impl::_alloc_counter;
             alglib_impl::ae_state_init(&_alglib_env_state);
             alglib_impl::ae_frame_make(&_alglib_env_state, &_frame_block);
-            alglib_impl::ae_smart_ptr_init(&ptr0, (void**)&p0, &_alglib_env_state, ae_true);
-            alglib_impl::ae_smart_ptr_init(&ptr1, (void**)&p1, &_alglib_env_state, ae_true);
-            alglib_impl::ae_shared_pool_init(&pool, &_alglib_env_state, ae_true);
-            _seedrec_init(&seed, &_alglib_env_state, ae_true);
+            alglib_impl::ae_smart_ptr_init(&ptr0, (void**)&p0, &_alglib_env_state);
+            alglib_impl::ae_smart_ptr_init(&ptr1, (void**)&p1, &_alglib_env_state);
+            alglib_impl::ae_shared_pool_init(&pool, &_alglib_env_state);
+            _seedrec_init(&seed, &_alglib_env_state);
             alglib_impl::ae_shared_pool_set_seed(&pool, &seed, sizeof(seed), _seedrec_init, _seedrec_init_copy, _seedrec_destroy, &_alglib_env_state);
             alglib_impl::ae_shared_pool_retrieve(&pool, &ptr0, &_alglib_env_state);
             alglib_impl::ae_shared_pool_retrieve(&pool, &ptr1, &_alglib_env_state);
@@ -1274,6 +1266,79 @@ int main()
         fflush(stdout);
         if( !issue528_passed )
             return 1;
+        
+        //
+        // Testing issue #591 (http://bugs.alglib.net/view.php?id=591)
+        // in copying of object containing shared pool as one of its
+        // fields.
+        //
+        // Unfixed ALGLIB crashes because of unneeded assertion in the
+        // ae_shared_pool_init_copy() function.
+        //
+        try
+        {
+            alglib::multilayerperceptron net0, net1;
+            alglib::real_1d_array x("[1,2]"), y0("[0,0]"), y1("[0,0]"), y2("[0,0]");
+            alglib::mlpcreate0(2, 2, net0);
+            alglib::mlpprocess(net0, x, y0);
+            
+            //
+            // Test assignment constructor
+            //
+            net1 = net0;
+            alglib::mlpprocess(net1, x, y1);
+            issue591_passed = issue591_passed && (fabs(y0[0]-y1[0])<1.0E-9) && (fabs(y0[1]-y1[1])<1.0E-9);
+            
+            //
+            // Test copy constructor
+            //
+            alglib::multilayerperceptron net2(net0);
+            alglib::mlpprocess(net2, x, y2);
+            issue591_passed = issue591_passed && (fabs(y0[0]-y2[0])<1.0E-9) && (fabs(y0[1]-y2[1])<1.0E-9);
+        }
+        catch(...)
+        { issue591_passed = false; }
+        printf(fmt_str, "* issue 591", issue591_passed ? "OK" : "FAILED");
+        fflush(stdout);
+        if( !issue591_passed )
+            return 1;
+        
+        //
+        // Task #594 (http://bugs.alglib.net/view.php?id=594) - additional
+        // test for correctness of copying of objects. When we copy ALGLIB
+        // object, indenendent new copy is created.
+        //
+        // This test checks both copying with copy constructor and assignment
+        // constructor
+        //
+        try
+        {
+            alglib::multilayerperceptron net0, net1;
+            alglib::real_1d_array x("[1,2]"), y0("[0,0]"), y1("[0,0]"), y2("[0,0]");
+            alglib::mlpcreate0(2, 2, net0);
+            alglib::mlpprocess(net0, x, y0);
+            
+            //
+            // Test assignment and copy constructors:
+            // * copy object with one of the constructors
+            // * process vector with original network
+            // * randomize original network
+            // * process vector with copied networks and compare
+            //
+            net1 = net0;
+            alglib::multilayerperceptron net2(net0);
+            alglib::mlprandomize(net0);
+            alglib::mlpprocess(net1, x, y1);
+            alglib::mlpprocess(net2, x, y2);
+            issue594_passed = issue594_passed && (fabs(y0[0]-y1[0])<1.0E-9) && (fabs(y0[1]-y1[1])<1.0E-9);
+            issue594_passed = issue594_passed && (fabs(y0[0]-y2[0])<1.0E-9) && (fabs(y0[1]-y2[1])<1.0E-9);
+        }
+        catch(...)
+        { issue594_passed = false; }
+        printf(fmt_str, "* issue 594", issue594_passed ? "OK" : "FAILED");
+        fflush(stdout);
+        if( !issue594_passed )
+            return 1;
     }
     
     //
@@ -1312,7 +1377,7 @@ int main()
                         c[i][j] = 0.0;
                     }
                 
-                t = alglib_impl::tickcount();
+                t = alglib_impl::_tickcount();
                 for(k=0; k<nrepeat; k++)
                     alglib::rmatrixgemm(
                         n, n, n,
@@ -1321,12 +1386,12 @@ int main()
                         b, 0, 0, (k/2)%2,
                         0.0,
                         c, 0, 0);
-                t = alglib_impl::tickcount()-t;
+                t = alglib_impl::_tickcount()-t;
                 perf0 = 1.0E-6*pow((double)n,3)*2.0*nrepeat/(0.001*t);
                 printf("* RGEMM-SEQ-%-4ld (MFLOPS)  %5.0lf\n", (long)n, (double)perf0);
                 
                 alglib::setnworkers(1);
-                t = alglib_impl::tickcount();
+                t = alglib_impl::_tickcount();
                 for(k=0; k<nrepeat; k++)
                     alglib::smp_rmatrixgemm(
                         n, n, n,
@@ -1335,12 +1400,12 @@ int main()
                         b, 0, 0, (k/2)%2,
                         0.0,
                         c, 0, 0);
-                t = alglib_impl::tickcount()-t;
+                t = alglib_impl::_tickcount()-t;
                 perf1 = 1.0E-6*pow((double)n,3)*2.0*nrepeat/(0.001*t);
                 printf("* RGEMM-MT1-%-4ld (MFLOPS)  %4.1lfx\n", (long)n, (double)(perf1/perf0));
                 
                 alglib::setnworkers(0);
-                t = alglib_impl::tickcount();
+                t = alglib_impl::_tickcount();
                 for(k=0; k<nrepeat; k++)
                     alglib::smp_rmatrixgemm(
                         n, n, n,
@@ -1349,7 +1414,7 @@ int main()
                         b, 0, 0, (k/2)%2,
                         0.0,
                         c, 0, 0);
-                t = alglib_impl::tickcount()-t;
+                t = alglib_impl::_tickcount()-t;
                 perf2 = 1.0E-6*pow((double)n,3)*2.0*nrepeat/(0.001*t);
                 printf("* RGEMM-MTN-%-4ld           %4.1lfx\n", (long)n, (double)(perf2/perf0));
                 
